@@ -16,10 +16,10 @@ use crate::buffer_::{
 };
 use crate::color::{self, IntoColor};
 use crate::error::{ImageError, ImageResult, ParameterError, ParameterErrorKind};
-// FIXME: These imports exist because we don't support all of our own color types.
-use crate::error::{ImageFormatHint, UnsupportedError, UnsupportedErrorKind};
 use crate::flat::FlatSamples;
-use crate::image::{GenericImage, GenericImageView, ImageDecoder, ImageFormat, ImageOutputFormat};
+use crate::image::{
+    GenericImage, GenericImageView, ImageDecoder, ImageEncoder, ImageFormat, ImageOutputFormat,
+};
 use crate::imageops;
 use crate::io::free_functions;
 use crate::math::resize_dimensions;
@@ -454,7 +454,7 @@ impl DynamicImage {
         }
     }
 
-    /// Return a reference to an 16bit RGB image
+    /// Return a reference to an 32bit RGB image
     pub fn as_rgb32f(&self) -> Option<&Rgb32FImage> {
         match *self {
             DynamicImage::ImageRgb32F(ref p) => Some(p),
@@ -662,6 +662,9 @@ impl DynamicImage {
     /// The image is scaled to the maximum possible size that fits
     /// within the bounds specified by `nwidth` and `nheight`.
     pub fn resize(&self, nwidth: u32, nheight: u32, filter: imageops::FilterType) -> DynamicImage {
+        if (nwidth, nheight) == self.dimensions() {
+            return self.clone();
+        }
         let (width2, height2) =
             resize_dimensions(self.width(), self.height(), nwidth, nheight, false);
 
@@ -829,14 +832,14 @@ impl DynamicImage {
             #[cfg(feature = "png")]
             image::ImageOutputFormat::Png => {
                 let p = png::PngEncoder::new(w);
-                p.encode(bytes, width, height, color)?;
+                p.write_image(bytes, width, height, color)?;
                 Ok(())
             }
 
             #[cfg(feature = "pnm")]
             image::ImageOutputFormat::Pnm(subtype) => {
-                let mut p = pnm::PnmEncoder::new(w).with_subtype(subtype);
-                p.encode(bytes, width, height, color)?;
+                let p = pnm::PnmEncoder::new(w).with_subtype(subtype);
+                p.write_image(bytes, width, height, color)?;
                 Ok(())
             }
 
@@ -851,6 +854,11 @@ impl DynamicImage {
         }
     }
 
+    /// Encode this image with the provided encoder.
+    pub fn write_with_encoder(&self, encoder: impl ImageEncoder) -> ImageResult<()> {
+        dynamic_map!(self, |ref p| p.write_with_encoder(encoder))
+    }
+
     /// Saves the buffer to a file at the path specified.
     ///
     /// The image format is derived from the file extension.
@@ -1070,17 +1078,6 @@ fn decoder_to_image<'a, I: ImageDecoder<'a>>(decoder: I) -> ImageResult<DynamicI
             let buf = image::decoder_to_vec(decoder)?;
             ImageBuffer::from_raw(w, h, buf).map(DynamicImage::ImageLumaA16)
         }
-
-        // An internal #[non_exhaustive]
-        #[allow(unreachable_patterns)]
-        _ => {
-            return Err(ImageError::Unsupported(
-                UnsupportedError::from_format_and_kind(
-                    ImageFormatHint::Unknown,
-                    UnsupportedErrorKind::Color(color_type.into()),
-                ),
-            ))
-        }
     };
 
     match image {
@@ -1172,7 +1169,7 @@ where
 /// This will lead to corrupted writers if the buffer contains
 /// malformed data.
 ///
-/// See [`ImageOutputFormat`](../enum.ImageOutputFormat.html) for
+/// See [`ImageOutputFormat`](enum.ImageOutputFormat.html) for
 /// supported types.
 ///
 /// Assumes the writer is buffered. In most cases,
@@ -1332,7 +1329,7 @@ mod test {
         // ensures that DynamicImage implements Default (if it didn't, this would cause a compile error).
         #[derive(Default)]
         struct Foo {
-            image: super::DynamicImage,
+            _image: super::DynamicImage,
         }
     }
 
